unit GeneratorXml;

interface

uses
  Classes, SysUtils, Source, Generator;

type
  { TGeneratorXml }

  TGeneratorXml = class(TGenerator)
  private
    procedure GenerateNodes(SourceNodes: TSourceNodes);
    procedure GenerateNode(SourceNode: TSourceNode);
  public
    procedure Generate; override;
    constructor Create; override;
  end;


implementation

{ TGeneratorXml }

procedure TGeneratorXml.GenerateNodes(SourceNodes: TSourceNodes);
var
  I: Integer;
begin
  for I := 0 to SourceNodes.Count - 1 do begin
    if SourceNodes[I] is TSourceNode then begin
      GenerateNode(TSourceNode(SourceNodes[I]));
    end else raise Exception.Create('Unsupported node type');
  end;
end;

procedure TGeneratorXml.GenerateNode(SourceNode: TSourceNode);
var
  I: Integer;
  Field: TField;
  Obj: TObject;
begin
  if SourceNode = nil then begin
  end else
  if SourceNode is TSourceNodes then begin
    GenerateNodes(TSourceNodes(SourceNode))
  end else
  if SourceNode is TSourceNode then begin
    AddTextLine('<' + SourceNode.ClassName + '>');
    Indent := Indent + 1;
    for I := 0 to SourceNode.FieldsCount - 1 do begin
      Field := SourceNode.GetField(I);
      if Field.DataType = dtObject then begin
        Obj := SourceNode.GetValueObject(I);
        if Obj is TSourceNode then
          GenerateNode(TSourceNode(Obj));
      end else
      if Field.DataType = dtString then begin
        AddTextLine('<' + Field.Name + '>' + SourceNode.GetValueString(I) + '<' + Field.Name + '>');
      end;
      Field.Free;
    end;
    Indent := Indent - 1;
    AddTextLine('</' + SourceNode.ClassName + '>');
  end else
    raise Exception.Create('Unsupported node type');
end;

procedure TGeneratorXml.Generate;
begin
  GenerateNode(Prog);
end;

constructor TGeneratorXml.Create;
begin
  inherited;
  Name := 'XML';
  FileExt := '.xml';
end;

end.

